"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateMockFrom = exports.mockFrom = exports.responseFrom = void 0;
const sdk_mapper_js_1 = require("./sdk-mapper.js");
const responseFrom = (oas, method) => {
    const statusCode = Object.keys(method.responses).find(x => x);
    const refResponse = method.responses[statusCode];
    const response = Object.prototype.hasOwnProperty.call(refResponse, '$ref') ? (0, sdk_mapper_js_1.traversePath)(refResponse['$ref'], oas) : refResponse;
    const contentType = Object.keys(response.content ?? '')[0];
    const content = response.content ? response.content[contentType] : undefined;
    return [statusCode, content?.example ?? {}];
};
exports.responseFrom = responseFrom;
const mockFrom = (spec) => ({
    name: spec.info.title.replace(/ /g, '') + 'Mock',
    paths: (0, sdk_mapper_js_1.pathsFrom)(spec).map(path => ({
        path: path.path,
        methods: (0, sdk_mapper_js_1.methodOperations)(path.definition).map(methodOperation => {
            const [statusCode, response] = (0, exports.responseFrom)(spec, methodOperation.definition);
            return {
                statusCode,
                response,
                method: methodOperation.method,
            };
        })
    }))
});
exports.mockFrom = mockFrom;
const generateMockFrom = (spec) => {
    const mock = (0, exports.mockFrom)(spec);
    return `
const ${mock.name} = (app: any, basePath: string) => {
  ${mock.paths.map(path => path.methods.map(method => `app.${method.method}(basePath + "${path.path.replace(/{([^/{}]+)}/g, ':$1')}", (req: any, res: any) => res.status(${method.statusCode}).json(${JSON.stringify(method.response)}));`).join("\n  ")).join("\n  ")}
}

export default ${mock.name};
`;
};
exports.generateMockFrom = generateMockFrom;
