"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathBuilder = void 0;
class PathBuilder {
    name;
    path = {};
    constructor(name) {
        this.name = name;
        const matches = name.matchAll(/{([^/]+)}/g);
        this.set('parameters', [...matches].map((match) => {
            const parameter = {
                name: match[1],
                in: 'path',
                required: true,
                schema: { type: "string" }
            };
            return parameter;
        }));
    }
    childPath(name, builder) {
        const pathBuilder = new PathBuilder(this.name + '/' + name);
        const childPaths = builder(pathBuilder);
        this.path = { ...this.path, ...childPaths.path };
        return this;
    }
    resource(name, builder) {
        return this.childPath(`{${name}}`, builder);
    }
    set(key, value) {
        this.path[this.name] = { ...(this.path[this.name] ?? {}), [key]: value };
        return this;
    }
    summary(summary) { return this.set('summary', summary); }
    description(description) { return this.set('description', description); }
    servers(...servers) {
        return this.set('servers', servers);
    }
    parameters(...parameters) { return this.set('parameters', parameters); }
    get(operationId, operation) {
        return this.set('get', { ...operation, operationId });
    }
    put(operationId, operation) {
        return this.set('put', { ...operation, operationId });
    }
    post(operationId, operation) {
        return this.set('post', { ...operation, operationId });
    }
    delete(operationId, operation) {
        return this.set('delete', { ...operation, operationId });
    }
    options(operationId, operation) {
        return this.set('options', { ...operation, operationId });
    }
    head(operationId, operation) {
        return this.set('head', { ...operation, operationId });
    }
    patch(operationId, operation) {
        return this.set('patch', { ...operation, operationId });
    }
    trace(operationId, operation) {
        return this.set('trace', { ...operation, operationId });
    }
}
exports.PathBuilder = PathBuilder;
