import { Path } from './path';
export class PathFinder {
    apiName;
    paths;
    constructor(apiName, paths = []) {
        this.apiName = apiName;
        this.paths = paths;
    }
    append(route, path, securitySchemes) {
        const [root, ...rest] = route.substring(1).split('/');
        const existing = this.paths.find(it => it.part === root);
        if (existing) {
            existing.append(rest, path, securitySchemes);
        }
        else {
            this.paths.push(new Path(root).append(rest, path, securitySchemes));
        }
        return this;
    }
    pathInfo() {
        return this.paths.reduce((paths, path) => ({ ...paths, ...path.pathInfo() }), {});
    }
    static from(openapi) {
        return Object.keys(openapi.paths).reduce((pathFinder, path) => {
            return pathFinder.append(path, openapi.paths[path], openapi.components?.securitySchemes);
        }, new PathFinder(openapi.info.title.replace(/\W+/g, ''), []));
    }
}
