import { methodFrom } from './method';
export class Path {
    part;
    paths;
    methods;
    parent;
    constructor(part, paths = [], methods = [], parent) {
        this.part = part;
        this.paths = paths;
        this.methods = methods;
        this.parent = parent;
    }
    operations() {
        const children = this.paths.flatMap(path => path.operations());
        const currentLevel = this.methods.map(method => ({ path: this, method }));
        return [...currentLevel, ...children];
    }
    append(route, path, securitySchemes) {
        if (route.length === 0) {
            const methods = ['get', 'put', 'post', 'delete', 'options', 'head', 'patch', 'trace'];
            Object.keys(path).filter(it => methods.includes(it)).forEach(verb => {
                this.methods.push(methodFrom(verb, path[verb]));
            });
        }
        else {
            const [root, ...rest] = route;
            const existing = this.paths.find(it => it.part === root);
            if (existing) {
                existing.append(rest, path, securitySchemes);
            }
            else {
                this.paths.push(new Path(root, [], [], this).append(rest, path, securitySchemes));
            }
        }
        return this;
    }
    pathInfo() {
        return {
            ['/' + this.part]: {
                security: this.methods.reduce((prev, method) => ({ ...prev, [method.method.toUpperCase()]: { scopes: method.scopes } }), {}),
                methods: this.methods.map(method => method.method.toUpperCase()),
                paths: this.paths.reduce((paths, path) => ({ ...paths, ...path.pathInfo() }), {})
            }
        };
    }
    pathString() {
        return (this.parent?.pathString() ?? '') + '/' + this.part;
    }
    capitilize(name) {
        return name.substring(0, 1).toUpperCase() + name.substring(1);
    }
    name() {
        return this.part.startsWith('{') ? ('By' + this.capitilize(this.part.substring(1, this.part.length - 1))) : this.capitilize(this.part);
    }
}
