import { JSONSchema } from 'json-schema-to-typescript';
import { OAS, OASComponents, OASEncoding, OASInfo, OASMedia, OASOAuthFlows, OASParameter, OASResponse, OASSecurity, OASSecurityScheme } from './oas';
import { PathBuilder } from './path-builder';
export type OASParts = {
    schemes: string[];
    apiIntegrations: string[];
};
export declare class OpenApiSpecificationBuilder<S extends {
    components: {
        schemas?: any;
        responses?: Record<string, any>;
    };
}, T extends OASParts = {
    schemes: [];
    apiIntegrations: [];
}> {
    oas: OAS;
    parts: OASParts;
    private constructor();
    private _defaultResponses;
    build(): OAS;
    withAWSCognitoSecurityScheme<K extends string>(key: K, userPoolEndpoint: string, clientId: string, identitySource?: string): OpenApiSpecificationBuilder<S, {
        schemes: [...T['schemes'], K];
        apiIntegrations: T['apiIntegrations'];
    }>;
    withAWSLambdaApiGatewayIntegration<K extends string>(key: K, functionUri: string, payloadFormatVersion?: '1.0' | '2.0', passthroughBehavior?: 'when_no_templates' | 'when_no_match' | 'never'): OpenApiSpecificationBuilder<S, {
        schemes: T['schemes'];
        apiIntegrations: [...T['apiIntegrations'], K];
    }>;
    get securitySchemes(): Record<T['schemes'][number], (scopes?: string[]) => OASSecurity>;
    get awsLambdaApiGatewayIntegration(): Record<T['apiIntegrations'][number], {
        '$ref': string;
    }>;
    withPath(name: string, builder: (path: PathBuilder, oas: this) => PathBuilder): this;
    jsonContent<K extends keyof S['components']['schemas']>(key: K, example?: any, examples?: Record<string, any>, encoding?: {
        [key: string]: OASEncoding;
    }): {
        'application/json': OASMedia;
    };
    textContent(example?: string, examples?: Record<string, string>, encoding?: {
        [key: string]: OASEncoding;
    }): {
        'application/text': OASMedia;
    };
    response(content: OASResponse['content'], description?: string, headers?: string[]): OASResponse;
    textResponse<K extends keyof S['components']['schemas']>(description?: string, headers?: string[], example?: any, examples?: Record<string, any>, encoding?: {
        [key: string]: OASEncoding;
    }): OASResponse;
    jsonResponse<K extends keyof S['components']['schemas']>(key: K, description?: string, headers?: string[], example?: any, examples?: Record<string, any>, encoding?: {
        [key: string]: OASEncoding;
    }): OASResponse;
    responseReference<K extends keyof S['components']['responses']>(key: K): {
        '$ref': K extends string ? `#/components/responses/${K}` : string;
    };
    media<K extends keyof S['components']['schemas']>(key: K, example?: any, examples?: Record<string, any>, encoding?: {
        [key: string]: OASEncoding;
    }): OASMedia;
    basicAuthScheme(description?: string): OASSecurityScheme;
    apiKeyScheme(in_: "header" | "query" | "cookie", name: string, description?: string): OASSecurityScheme;
    bearerScheme(description?: string): OASSecurityScheme;
    oAuth2Scheme(flows: OASOAuthFlows, description?: string): OASSecurityScheme;
    openIdConnectScheme(openIdConnectUrl: string, description?: string): OASSecurityScheme;
    query(name: string, required?: boolean, multiple?: boolean): OASParameter;
    header(name: string, required?: boolean): OASParameter;
    parameter(name: string, location: OASParameter['in'], required?: boolean, schema?: JSONSchema): OASParameter;
    reference<K extends keyof S['components']['schemas']>(key: K): {
        '$ref': K extends string ? `#/components/schemas/${K}` : string;
    };
    componentReference<K extends keyof S['components'], T extends keyof S['components'][K]>(componentKey: K, key: T): {
        '$ref': K extends string ? T extends string ? `#/components/${K}/${T}` : string : string;
    };
    defaultResponses(itemBuilder: (builder: this) => Exclude<OASComponents["responses"], undefined>): this;
    addComponent<K extends keyof OASComponents, B extends (builder: this) => OASComponents[K]>(location: K, itemBuilder: B): B extends (builder: any) => infer R ? OpenApiSpecificationBuilder<S & {
        components: {
            [k in K]: R;
        };
    }> : never;
    private modifyOperationWithDefaultResponses;
    private addDefaultResponsesToPath;
    private addDefaultResponses;
    add<K extends keyof OAS>(location: K, itemBuilder: (builder: this) => OAS[K]): this;
    static create<S extends {
        components: {
            schemas: any;
            responses?: Record<string, any>;
        };
    }>(schemas: S, info: OASInfo): OpenApiSpecificationBuilder<S>;
}
